/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.data;

import amberstone.lang.KeyFunction;

public class BytesKey implements Key, KeyFunction
{
	private ByteArray byteValue = new ByteArray();
	private int hashCode;
	private int type;
	private int strength;

	public BytesKey(ByteArray byteValue)
	{
		this.byteValue = byteValue;
	}

	public BytesKey(int size)
	{
		super();
		this.byteValue = new ByteArray(size);
	}

	public void setType(int type)
	{
		this.type = type;
	}

	public int getType()
	{
		return this.type;
	}

	public void setStrength(int strength)
	{
		this.strength = strength;
	}

	public Key cloneKey()
	{
		BytesKey klone = new BytesKey(byteValue.getBuffer().length);
		klone.setType(this.type);
		klone.copyKey(this);
		return klone;
	}

	public Key getKey()
	{
		return this;
	}

	public boolean copyKey(Key key)
	{
		if(this.type == key.getType())
		{
			this.hashCode = key.hashCode();

			ByteArray keyBytes = key.byteValue();
			byte[] keyBuffer = keyBytes.getBuffer();

			if(keyBuffer.length <= byteValue.length)
			{
				byteValue.writeBytes(keyBytes);
			}
			else
			{
				byte[] buffer = new byte[keyBytes.length];
				byteValue.set(buffer, 0, 0);
				byteValue.writeBytes(keyBytes);
			}
			byteValue.reset();

			return true;
		}

		return false;
	}

	public boolean equalsKey(Key k)
	{
		if(this.hashCode == k.hashCode())
		{
			ByteArray compare = k.byteValue();
			int compareLength = compare.length;
			if(byteValue.length == compareLength)
			{
				byte[] compBuf = compare.getBuffer();
				int j = compare.offset;
				int max = j+compareLength;

				byte[] buf = byteValue.getBuffer();
				int i = byteValue.offset;
				int s = strength;

				while( j < max && buf[i] == compBuf[j])
				{
					i+=s;
					j+=s;
				}

				if(j >= max)
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		return false;
	}


	public boolean isNull()
	{
		return false;
	}

	public int hashCode()
	{
		return this.hashCode;
	}

	public int intValue()
	{
		return 0;
	}

	public long longValue()
	{
		return 0L;
	}

	public void setByteValue(ByteArray byteValue)
	{
		this.byteValue = byteValue;
	}

	public ByteArray byteValue()
	{
		return this.byteValue;
	}

	public String toString()
	{
		return byteValue.toString();
	}
}