/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.data;

import amberstone.lang.*;

public abstract class Field implements Comparable
{
	protected boolean hash;
	protected String name;
	protected int fieldOffset;
	protected int columnIndex;
	protected int type;
	protected ByteArray buffer = new ByteArray();

	public Field(int type,
				 String name,
				 int columnIndex)
	{
		this.name = name;
		this.type = type;
		this.columnIndex = columnIndex;
	}

	public ByteArray getBuffer()
	{
		return this.buffer;
	}

	public void setBuffer(ByteArray buffer)
	{
		this.buffer = buffer;
	}

	public void reset()
	{
		buffer.clear();
	}

	public void setHash(boolean hash)
	{
		this.hash = hash;
	}

	public boolean equals(Object o)
	{
		Field f = (Field)o;
		if(f.columnIndex == columnIndex)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	public int compareTo(Object o)
	{
		Field f = (Field)o;
		if(f.columnIndex == columnIndex)
		{
			return 0;
		}
		else if(f.columnIndex < columnIndex)
		{
			return 1;
		}
		else
		{
			return -1;
		}
	}

	public void setColumnIndex(int columnIndex)
	{
		this.columnIndex = columnIndex;
	}

	public int getColumnIndex()
	{
		return this.columnIndex;
	}

	public String getName()
	{
		return this.name;
	}

	public void initBuffer(byte[] bytes, int offset, int length)
	{
		this.buffer.set(bytes, offset, length);
	}

	//public abstract void setPresent(boolean p);
	public abstract Field cloneField();
	public abstract void clear();
	public abstract int maxLength();
	public abstract int minLength();
	public abstract int prepareWrite() throws FieldException;
	public abstract int writeField();
	public abstract void readField();
	public abstract int getInt(Function caller) throws FunctionException;
	public abstract long getLong(Function caller) throws FunctionException;
	public abstract ByteArray getBytes(Function caller) throws FunctionException;
	public abstract int getTime(Function caller) throws FunctionException;
	public abstract Key getKey();
	public abstract String signiture();
}