/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.data;

import amberstone.lang.KeyFunction;
import amberstone.dictionary.Dictionary;

public class IntKey implements Key, KeyFunction
{
	protected int type;
	protected int intValue;
	protected int hashCode;
	protected Dictionary dictionary;

	public IntKey()
	{

	}

	public void setDictionary(Dictionary dictionary)
	{
		this.dictionary = dictionary;
	}

	public IntKey(int intValue)
	{
		this.type = -1;
		this.intValue = intValue;
		this.hashCode = intValue;
	}

	public void setInt(int i)
	{
		this.hashCode = this.intValue = i;
	}

	public boolean equalsKey(Key key)
	{
		return this.intValue == key.intValue();
	}

	public Key cloneKey()
	{
		IntKey klone = new IntKey();
		klone.setType(type);
		klone.setIntValue(intValue);
		klone.setHashCode(hashCode);
		klone.setDictionary(this.dictionary);
		return klone;
	}

	public boolean isNull()
	{
		return false;
	}

	public Key getKey()
	{
		return this;
	}

	public int getType()
	{
		return this.type;
	}

	public void setType(int type)
	{
		this.type = type;
	}

	public int intValue()
	{
		return this.intValue;
	}

	public void setIntValue(int intValue)
	{
		this.intValue = intValue;
	}

	public void setHashCode(int hashCode)
	{
		this.hashCode = hashCode;
	}

	public long longValue()
	{
		return (long)this.intValue;
	}

	public ByteArray byteValue()
	{
		return null;
	}

	public int hashCode()
	{
		return this.hashCode;
	}

	public String toString()
	{
		if(dictionary == null)
		{
			return Integer.toString(intValue);
		}
		else
		{
			try
			{
				ByteArray b = dictionary.getTerm(intValue);
				return b.toString();
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}

		return null;
	}
}