/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.data;

import amberstone.lang.KeyFunction;

public class LongKey implements Key, KeyFunction
{
	protected int type;
	protected long longValue;
	protected int hashCode;

	public LongKey()
	{

	}

	public LongKey(long l)
	{
		this.type = -1;
		this.longValue = l;
		this.hashCode = ((((int)(this.longValue>>>32))+((int)this.longValue))) & 0x7FFFFFFF;
	}

	public boolean equalsKey(Key key)
	{
		if(this.longValue == key.longValue())
		{
			return true;
		}

		return false;
	}

	public Key cloneKey()
	{
		LongKey klone = new LongKey();
		klone.setType(type);
		klone.setLongValue(longValue);
		klone.setHashCode(hashCode);
		return klone;
	}

	public boolean isNull()
	{
		return this.longValue == 0;
	}

	public Key getKey()
	{
		return this;
	}

	public int getType()
	{
		return this.type;
	}

	public void setType(int type)
	{
		this.type = type;
	}

	public int intValue()
	{
		return (int)this.longValue;
	}

	public long longValue()
	{
		return this.longValue;
	}

	public void setLongValue(long longValue)
	{
		this.longValue = longValue;
		this.hashCode = ((((int)(this.longValue>>>32))+((int)this.longValue))) & 0x7FFFFFFF;
	}

	public void setHashCode(int hashCode)
	{
		this.hashCode = hashCode;
	}

	public ByteArray byteValue()
	{
		return null;
	}

	public String toString()
	{
		return Long.toString(this.longValue);
	}

	public int hashCode()
	{
		return this.hashCode;
	}
}