/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.data;

import amberstone.lang.KeyFunction;

public class StringKey implements Key, KeyFunction
{
	private int hashCode;
	public String value;
	private ByteArray bytes;
	private boolean nullCheck;

	public StringKey()
	{

	}

	public StringKey(String value)
	{
		this.hashCode = value.hashCode() & 0x7FFFFFFF;
		this.value = value;
		this.bytes = new ByteArray(value);
		if(value.length() == 0)
		{
			this.nullCheck = true;
		}
	}

	public boolean isNull()
	{
		return nullCheck;
	}

	public Key cloneKey()
	{
		return this;
	}

	public Key getKey()
	{
		return this;
	}

	public boolean equalsKey(Key k)
	{
		return this == k;
	}

	public ByteArray getBytes()
	{
		return this.bytes;
	}

	public String toString()
	{
		return value;
	}

	public int getType()
	{
		return -1;
	}

	public int intValue()
	{
		return -1;
	}

	public long longValue()
	{
		return -1L;
	}

	public ByteArray byteValue()
	{
		return bytes;
	}

	public int hashCode()
	{
		return hashCode;
	}
}