/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.data.fields;

import amberstone.data.*;
import amberstone.lang.*;

public class Int1 extends Int
{


	public Int1(int type,
				String name,
				int index,
				IntFunction loadFunc)
	{
		super(type, name, index, loadFunc);
	}

	public Field cloneField()
	{
		Int1 i1 =  new Int1(this.type, this.name, this.columnIndex, this.loadFunc);
		i1.setDictionary(dictionary);
		return i1;
	}

	public int maxLength()
	{
		return 1;
	}

	public int minLength()
	{
		return 1;
	}

	public int prepareWrite() throws FieldException
	{
		try
		{
			if(this.dictionary != null)
			{
				ByteArray b = this.bytesFunc.getBytes();
				this.intValue = this.dictionary.putTerm(b);
			}
			else
			{
				this.intValue = loadFunc.getInt();
			}

			if(this.intValue > 255 || this.intValue < 0)
			{
				throw new Exception("int1 must be between 0 and 255:"+this.intValue);
			}
		}
		catch(Exception e)
		{
			throw new FieldException(this.name, e);
		}

		if(buffer.getPointer()+1 < buffer.getBuffer().length)
		{
			return 1;
		}
		else
		{
			return -1;
		}
	}

	public void init(int recordsPerBlock)
	{
		byte[] buf = new byte[recordsPerBlock];
		buffer.set(buf,0,0);
	}

	public int writeField()
	{
		buffer.writeInt1(this.intValue);
		return 1;
	}

	public Key getKey()
	{
		return this;
	}

	public void readField()
	{
		this.intValue = buffer.length>0 ? buffer.readInt1() : 0;
	}

	public int hashCode()
	{
		return this.intValue;
	}
}