/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.data.fields;

import amberstone.data.*;
import amberstone.lang.*;

public class Int3 extends Int
{



	public Int3(int type,
				String name,
				int index,
				IntFunction loadFunc)
	{
		super(type, name, index, loadFunc);
	}

	public Field cloneField()
	{
		Int3 i3 = new Int3(this.type, this.name, this.columnIndex, this.loadFunc);
		i3.setDictionary(this.dictionary);
		return i3;
	}

	public int maxLength()
	{
		return 3;
	}

	public int minLength()
	{
		return 3;
	}

	public int prepareWrite() throws FieldException
	{
		try
		{

			if(this.dictionary != null)
			{
				ByteArray b = this.bytesFunc.getBytes();
				this.intValue = this.dictionary.putTerm(b);
			}
			else
			{
				this.intValue = loadFunc.getInt();
			}

			if(this.intValue > 16777215 || this.intValue < 0)
			{
				throw new Exception("int3 must be between 0 and 16,777,215:"+this.intValue);
			}
		}
		catch(Exception e)
		{
			throw new FieldException(this.name, e);
		}

		if(buffer.getPointer()+3 < buffer.getBuffer().length)
		{
			return 3;
		}
		else
		{
			return -1;
		}
	}

	public Key getKey()
	{
		return this;
	}

	public int writeField()
	{
		buffer.writeInt3(this.intValue);
		return 3;
	}

	public void init(int recordsPerBlock)
	{
		byte[] buf = new byte[recordsPerBlock*3];
		buffer.set(buf, 0, 0);
	}


	public void readField()
	{
		this.intValue = buffer.length>0 ? buffer.readInt3() : 0;
	}

	public int hashCode()
	{
		return this.intValue;
	}


}