/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.data.fields;

import amberstone.data.*;
import amberstone.lang.*;


public class Int4 extends Int
{

	public Int4(int type,
				String name,
				int index,
				IntFunction loadFunc)
	{
		super(type, name, index, loadFunc);

	}

	public Field cloneField()
	{
		Int4 i4 =  new Int4(this.type, this.name, this.columnIndex, this.loadFunc);
		i4.setDictionary(this.dictionary);
		return i4;
	}

	public int maxLength()
	{
		return 4;
	}

	public int minLength()
	{
		return 4;
	}

	public int prepareWrite() throws FieldException
	{
		try
		{
			if(this.dictionary != null)
			{
				ByteArray b = this.bytesFunc.getBytes();
				this.intValue = this.dictionary.putTerm(b);
			}
			else
			{
				this.intValue = loadFunc.getInt();
			}
		}
		catch(Exception e)
		{
			throw new FieldException(this.name, e);
		}

		if(buffer.getPointer()+4 < buffer.getBuffer().length)
		{
			return 4;
		}
		else
		{
			return -1;
		}
	}

	public Key getKey()
	{
		if(!this.hash)
		{
			this.hashCode = this.intValue & 0x7FFFFFFF;
			hash = true;
		}

		return this;
	}

	public int writeField()
	{
		//System.out.println("Write int:"+this.intValue);
		buffer.writeInt(this.intValue);
		return 4;
	}

	public void init(int recordsPerBlock)
	{
		byte[] buf = new byte[recordsPerBlock*4];
		buffer.set(buf, 0, 0);
	}

	public void readField()
	{
		this.intValue = buffer.length >0 ? buffer.readInt() : 0;
		this.hash = false;
	}

	public int hashCode()
	{
		return this.hashCode;
	}
}