/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.data.fields;

import amberstone.data.*;
import amberstone.data.LongKey;
import amberstone.lang.*;

public class LongInt extends Field implements Key
{
	private LongFunction loadFunc;
	protected long longValue;
	protected int hashCode;

	public LongInt(int type,
				   String name,
				   int index,
				   LongFunction loadFunc)
	{
		super(type, name, index);
		this.loadFunc = loadFunc;
	}

	public void clear()
	{
		this.longValue = this.hashCode = 0;
	}

	public void setLoadFunction(LongFunction loadFunc)
	{
		this.loadFunc = loadFunc;
	}

	public Field cloneField()
	{
		return new LongInt(this.type, this.name, this.columnIndex, this.loadFunc);
	}

	public boolean equalsKey(Key k)
	{
		if(this.type == k.getType() && this.longValue == k.longValue())
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	public Key cloneKey()
	{
		LongKey klone = new LongKey();
		klone.setType(type);
		klone.setLongValue(longValue);
		klone.setHashCode(hashCode);
		return klone;
	}

	public boolean isNull()
	{
		return false;
	}

	public int hashCode()
	{
		return this.hashCode;
	}

	public int getType()
	{
		return this.type;
	}

	public int intValue()
	{
		return (int)this.longValue;
	}

	public void setLongValue(long longValue)
	{
		this.longValue = longValue;
	}

	public long longValue()
	{
		return this.longValue;
	}

	public ByteArray byteValue()
	{
		return null;
	}

	public int maxLength()
	{
		return 8;
	}

	public int minLength()
	{
		return 8;
	}

	public int prepareWrite() throws FieldException
	{
		try
		{
			this.longValue = this.loadFunc.getLong();
		}
		catch(Exception e)
		{
			throw new FieldException(this.name, e);
		}

		if(buffer.getPointer()+8 < buffer.getBuffer().length)
		{
			return 8;
		}
		else
		{
			return -1;
		}
	}

	public int writeField()
	{
		buffer.writeLong(this.longValue);
		return 8;
	}

	public void readField()
	{
		this.hash = false;
		this.longValue = buffer.length > 0 ? buffer.readLong() : 0;
	}

	public void init(int recordsPerBlock)
	{
		byte[] buf = new byte[recordsPerBlock*8];
		buffer.set(buf, 0, 0);
	}

	public Key getKey()
	{
		if(!hash)
		{
			this.hashCode  = ((((int)(this.longValue>>>32))+((int)this.longValue))) & 0x7FFFFFFF;
			hash = true;
		}

		return this;
	}

	public long getLong(Function caller)
	{
		return this.longValue;
	}

	public int getTime(Function caller) throws FunctionException
	{
		throw new FunctionException("Time not supported for long type.", caller);
	}

	public ByteArray getBytes(Function caller) throws FunctionException
	{
		throw new FunctionException("Bytes not supported for long type.", caller);
	}

	public int getInt(Function caller) throws FunctionException
	{
		throw new FunctionException("int not supported for long type.", caller);
	}

	public String signiture()
	{
		return this.name+"."+maxLength();
	}

	public String toString()
	{
		return Long.toString(this.longValue);
	}
}