/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.dictionary;

import java.io.IOException;

import amberstone.data.ByteArray;
import amberstone.util.MutableInteger;
import amberstone.util.LRUCache;
import amberstone.resources.ResourceIterator;
import amberstone.lang.DictionaryFunction;

public class CacheDictionary implements Dictionary,
										DictionaryFunction
{
	private Dictionary dictionary;
	private LRUCache cache;
	private MutableInteger ikey = new MutableInteger();
	private long hits;
	private long misses;

	public CacheDictionary(Dictionary dictionary, int cacheSize)
	{
		this.dictionary   = dictionary;
		this.cache  = new LRUCache(((int)(cacheSize*1.1)), 1.0F, cacheSize);
	}

	public Dictionary getDictionary()
	{
		return this;
	}

	public void clear()
	{
		hits = misses = 0;
		this.cache = null;
	}

	public long getHits()
	{
		return this.hits;
	}

	public long getMisses()
	{
		return this.misses;
	}

	public int putTerm(ByteArray term)
		throws Exception
	{
		MutableInteger id = (MutableInteger)cache.get(term);

		if(id == null)
		{
			term = term.cloneBytes();
			int i = dictionary.putTerm(term);
			cache.put(term, new MutableInteger(i));
			return i;
		}
		else
		{
			return id.getInt();
		}
	}


	public String getName()
	{
		return this.dictionary.getName();
	}

	public ByteArray getTerm(int id) throws Exception
	{
		ikey.setInt(id);
		ByteArray bytes = (ByteArray)cache.get(ikey);
		if(bytes == null)
		{
			bytes = dictionary.getTerm(id);
			if(bytes != null)
				cache.put(new MutableInteger(id), bytes);
		}

		return bytes;
	}

	public ResourceIterator terms() throws IOException
	{
		return dictionary.terms();

	}
}

