/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.dictionary;

import amberstone.lang.DictionaryFunction;
import amberstone.lang.FunctionException;
import amberstone.data.ByteArray;
import amberstone.resources.ResourceIterator;

import java.io.IOException;

public class Segments implements Dictionary
{
	private Dictionary[] segments;

	public Segments(Object[] params) throws FunctionException
	{
		segments = new Dictionary[params.length];
		for(int i=0; i<params.length; i++)
		{
			segments[i] = ((DictionaryFunction)params[i]).getDictionary();
		}
	}

	public ResourceIterator terms() throws IOException
	{
		return new SegmentsIterator(segments);
	}

	public String getName()
	{
		return null;
	}

	public int putTerm(ByteArray term)
			throws Exception
	{
		int index = term.hashCode() % segments.length;
		Dictionary segment = segments[index];
		int i = segment.putTerm(term);
		i = i << 7;
		i = i+index;
		return i;
	}

	public ByteArray getTerm(int i)
			throws Exception
	{
		int index = i & 0x0000007F;
		i = (i >>> 7) & 0x01ffffff;
		return segments[index].getTerm(i);
	}
}