/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.ByteArray;

public class Before extends Function implements BytesFunction, IntFunction, LongFunction
{
	private ByteArray slice = new ByteArray();
	private BytesFunction sourceBytesFunc;
	private ByteArray startBytes;
	private boolean takeAll = false;

	public static final FunctionType[][] allow = { { FunctionType.BYTES_FUNCTION, FunctionType.STRING_LITERAL},
	                                               { FunctionType.BYTES_FUNCTION, FunctionType.STRING_LITERAL, FunctionType.BOOLEAN_LITERAL} };

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params) throws ParameterException
	{
		this.sourceBytesFunc  = (BytesFunction)params[0];
		this.startBytes       = new ByteArray(params[1].toString());
		if(params.length == 3)
		{
			this.takeAll = ((BooleanLiteral)params[2]).getBoolean();
		}
		else
		{
			this.takeAll = false;
		}
	}

	public ByteArray getBytes() throws FunctionException
	{
		ByteArray sourceBytes = this.sourceBytesFunc.getBytes();
		int startIndex = sourceBytes.indexOf(this.startBytes);
		int length = 0;

		if(startIndex == -1)
		{
			if(takeAll)
			{
				return sourceBytes;
			}
		}
		else
		{
			length = startIndex;
		}

		slice.set(sourceBytes.getBuffer(), sourceBytes.getOffset(), length);
		return slice;
	}

	public int getInt() throws FunctionException
	{
		return this.getBytes().parseInt();
	}

	public long getLong() throws FunctionException
	{
		return this.getBytes().parseLong();
	}
}