/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.Key;
import amberstone.data.IntKey;
import amberstone.data.ByteArray;

public class Choose extends Function implements KeyFunction,
						                        IntFunction,
						                        LongFunction,
						                        BytesFunction
{
	private When[] ifs;

	private static final NumberLiteral zero = new NumberLiteral(0D);
	private static final ByteArray nullBytes = new ByteArray("");
	private int numTests;

	public FunctionType[][] strict()
	{
		return null;
	}

	public void params(Function[] params)
	{
		numTests = params.length;
		ifs = new When[numTests];

		for(int i=0; i<params.length; i++)
		{
			ifs[i] = (When)params[i];
		}
	}

	public Key getKey() throws FunctionException
	{
		for(int i=0; i<numTests; i++)
		{
			When test = ifs[i];
			if(test.pass())
			{
				return test.getKey();
			}
		}

		return zero.getKey();
	}

	public int getInt()  throws FunctionException
	{
		for(int i=0; i<numTests; i++)
		{
			When test = ifs[i];
			if(test.pass())
			{
				return test.getInt();
			}
		}

		return 0;
	}

	public long getLong() throws FunctionException
	{
		for(int i=0; i<numTests; i++)
		{
			When test = ifs[i];
			if(test.pass())
			{
				return test.getLong();
			}
		}
		return 0l;
	}

	public ByteArray getBytes() throws FunctionException
	{
		for(int i=0; i<numTests; i++)
		{
			When test = ifs[i];
			if(test.pass())
			{
				return test.getBytes();
			}
		}
		return nullBytes;
	}
}