/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.ByteArray;
import amberstone.dictionary.Dictionary;

public class Deflate extends Function implements IntFunction
{
	private BytesFunction bytesFunc;
	private DictionaryFunction dictionaryFunc;

	private static final FunctionType[][] allow = {{FunctionType.BYTES_FUNCTION, FunctionType.DICTIONARY_FUNCTION}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params)
	{
		this.bytesFunc = (BytesFunction)params[0];
		this.dictionaryFunc = (DictionaryFunction)params[1];
	}

	public int getInt() throws FunctionException
	{
		ByteArray term = this.bytesFunc.getBytes();

		try
		{
			Dictionary dictionary = this.dictionaryFunc.getDictionary();
			if(term.isNull())
			{
				return 0;
			}
			else
			{
				int i = dictionary.putTerm(term);
				return i;
			}
		}
		catch(Exception e)
		{
			throw new FunctionException("Error deflating term:"+ getErrorTerm(term), this, e);
		}
	}

	public String getErrorTerm(ByteArray term)
	{
		try
		{
			return term.toString();
		}
		catch(Exception e)
		{
			return "N/A";
		}
	}
}
