/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.ByteArray;
import amberstone.util.MutableInteger;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Map;
import java.util.HashMap;

public class DeflateTime
{
	private ByteArray slice = new ByteArray();
	private int YYYYStart;
	private int MMStart;
	private int DDStart;
	private int hhStart;
	private int mmStart;
	private int ssStart;

	private int currentDay;
	private int base;

	private boolean twoDigitMonth;
	private final Map<ByteArray, MutableInteger> months = new HashMap();

	private Calendar cal = new GregorianCalendar();

	private static final FunctionType[][] allow = {{FunctionType.STRING_LITERAL}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public DeflateTime(String template) throws ParameterException
	{
		this.YYYYStart = template.indexOf("YYYY");
		if(YYYYStart < 0)
		{
			throw new ParameterException(null, "Function template does not contain 'YYYY'");
		}

		this.MMStart = template.indexOf("MM");
		if(MMStart > 0)
		{
			this.twoDigitMonth = true;
		}
		else
		{
			this.MMStart = template.indexOf("Mon");

			if(MMStart < 0)
			{
				throw new ParameterException(null, "Unsupport month format");
			}
			else
			{
				this.twoDigitMonth = false;
				months.put(new ByteArray("jan"), new MutableInteger(0));
				months.put(new ByteArray("feb"), new MutableInteger(1));
				months.put(new ByteArray("mar"), new MutableInteger(2));
				months.put(new ByteArray("apr"), new MutableInteger(3));
				months.put(new ByteArray("may"), new MutableInteger(4));
				months.put(new ByteArray("jun"), new MutableInteger(5));
				months.put(new ByteArray("jul"), new MutableInteger(6));
				months.put(new ByteArray("aug"), new MutableInteger(7));
				months.put(new ByteArray("sep"), new MutableInteger(8));
				months.put(new ByteArray("oct"), new MutableInteger(9));
				months.put(new ByteArray("nov"), new MutableInteger(10));
				months.put(new ByteArray("dec"), new MutableInteger(11));
			}
		}

		this.DDStart = template.indexOf("DD");
		if(DDStart < 0)
		{
			throw new ParameterException(null, "Function template does not contain 'DD'");
		}

		this.hhStart = template.indexOf("hh");
		if(hhStart < 0)
		{
			throw new ParameterException(null, "Function template does not contain 'hh'");
		}

		this.mmStart = template.indexOf("mm");
		if(mmStart < 0)
		{
			throw new ParameterException(null, "Function template does not contain 'mm'");
		}

		this.ssStart = template.indexOf("ss");
		if(ssStart < 0)
		{
			throw new ParameterException(null, "Function template does not contain 'ss'");
		}
	}

	public int getTime(ByteArray bytes) throws FunctionException
	{
		byte[] buf = bytes.getBuffer();
		int offset = bytes.getOffset();

		slice.set(buf, offset+YYYYStart, 4);
		int year = slice.parseInt();

		int month = 0;
		if(this.twoDigitMonth)
		{
			slice.set(buf, offset+MMStart, 2);
			month = slice.parseInt();
		}
		else
		{
			slice.set(buf, offset+MMStart, 3);
			slice.toLowerCase();
			month = months.get(slice).getInt();
		}

		slice.set(buf, offset+DDStart, 2);
		int day = slice.parseInt();

		slice.set(buf, offset+hhStart, 2);
		int hour = slice.parseInt();
		//System.out.println(hour);

		slice.set(buf, offset+mmStart, 2);
		int minute = slice.parseInt();

		slice.set(buf, offset+ssStart, 2);
		int second = slice.parseInt();


		if(day != currentDay)
		{
			cal.clear();
			cal.set(year, month, day, 0, 0, 0);
			base = (int)(cal.getTimeInMillis()/1000);
			//System.out.println(base);
			currentDay = day;
		}

		//System.out.println(base+(hour*3600)+(minute*60)+second);

		return base+(hour*3600)+(minute*60)+second;
	}
}