/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.system.*;
import java.io.StringWriter;
import java.io.PrintWriter;

public class FunctionException extends SystemException
{
	private Function thrower;
	private String message;
	private String context;

	public FunctionException(String message, Function thrower)
	{
		super("");
		this.message = message;
		this.thrower = thrower;
	}

	public FunctionException(String message, Function thrower, Exception e)
	{
		super("", e);
		this.message = message;
		this.thrower = thrower;
	}

	public void setContext(String context)
	{
		this.context = context;
	}

	public void log(int logLevel, Logger logger)
	{
		logger.ln();
		logger.timestamp();
		logger.log("Function Exception");
		logger.ln();

		if(getMessage() != null)
		{
			logger.log(getMessage());
		}
	}

	public String getMessage()
	{
		try
		{
			StringWriter st = new StringWriter();
			PrintWriter printer = new PrintWriter(st);

			if(context != null)
			{
				printer.println(this.context);
			}

			printer.println("Exception in function: "+ thrower.getCallName());

			printer.println(this.message);
			printer.println(thrower.getCallTrace());
			Throwable t = getCause();
			if(t != null)
			{
				printer.println("");
				printer.println("Cause:");
				printer.println(t.getMessage());
			}

			return st.toString();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}
}

