/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.*;

public class Fuse extends Function implements KeyFunction, Dimension, LongFunction
{
	private LongKey key = new LongKey();
	private IntFunction intFunc1;
	private IntFunction intFunc2;
	private int shift;

	private static final FunctionType[][] allow = {{FunctionType.INT_FUNCTION, FunctionType.INT_FUNCTION}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params) throws ParameterException
	{
		this.intFunc1 = (IntFunction)params[0];
		this.intFunc2 = (IntFunction)params[1];
	}

	public String getDimension()
	{
		return "fuse";
	}

	public Key getKey()
	{
		try
		{
			long l = (((long)intFunc1.getInt())<<32)+((long)intFunc2.getInt());
			key.setLongValue(l);
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return new StringKey("");
		}
		return key;
	}

	public long getLong()
	{
		try
		{
			return (((long)intFunc1.getInt())<<32)+((long)intFunc2.getInt());
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return 0L;
		}
	}



}