/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.*;

public class Not extends Function implements BooleanFunction, IntFunction, KeyFunction
{
	private BooleanFunction booleanFunc;
	private IntFunction intFunc1;
	private IntFunction intFunc2;
	private IntKey ikey = new IntKey();


	private static final FunctionType[][] allowed = {{FunctionType.BOOLEAN_FUNCTION},
													 {FunctionType.INT_FUNCTION, FunctionType.INT_FUNCTION}};


	public FunctionType[][] strict()
	{
		return allowed;
	}

	public void params(Function[] params) throws ParameterException
	{
		if(params[0] instanceof BooleanFunction)
		{
			booleanFunc = (BooleanFunction)params[0];
		}

		if(params[0] instanceof IntFunction)
		{
			intFunc1 = (IntFunction)params[0];
			intFunc2 = (IntFunction)params[1];
		}
	}

	public boolean getBoolean() throws FunctionException
	{
		return !booleanFunc.getBoolean();
	}

	public int getInt() throws FunctionException
	{
		return this.intFunc1.getInt() ^ this.intFunc2.getInt();
	}

	public Key getKey() throws FunctionException
	{
		ikey.setInt(this.intFunc1.getInt() ^ this.intFunc2.getInt());
		return ikey;
	}
}