/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.ByteArray;

public class Slice extends Function implements BytesFunction, IntFunction, LongFunction
{
	private ByteArray slice = new ByteArray();
	private BytesFunction func;
	private IntFunction offsetFunc;
	private IntFunction numFunc;

	private static final FunctionType[][] allow = {{FunctionType.BYTES_FUNCTION, FunctionType.INT_FUNCTION, FunctionType.INT_FUNCTION}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params) throws ParameterException
	{
		this.func          = (BytesFunction)params[0];
		this.offsetFunc    = (IntFunction)params[1];
		this.numFunc       = (IntFunction)params[2];
	}

	public ByteArray getBytes() throws FunctionException
	{
		ByteArray bytes = func.getBytes();
		int offset = offsetFunc.getInt();
		int num = numFunc.getInt();

		if(offset > bytes.length || offset < 0)
		{
			offset = bytes.length;
		}

		int max = bytes.length-offset;

		if(num > max)
		{
			num = max;
		}

		this.slice.set(bytes.getBuffer(), bytes.offset+offset, num);
		return slice;
	}

	public int getInt() throws FunctionException
	{
		return this.getBytes().parseInt();
	}

	public long getLong() throws FunctionException
	{
		return this.getBytes().parseLong();
	}
}