/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.system.Logger;
import amberstone.system.Config;
import amberstone.resources.*;
import amberstone.data.Key;
import amberstone.data.ByteArray;
import amberstone.data.StringKey;
import amberstone.util.Primes;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

public class Switch extends Function implements KeyFunction
{
	private IntFunction intFunc;
	private ResourceFunction resourceFunc;
	private SwitchKey[] skeys;
	private int size;
	private Key nullKey = new StringKey("");

	private final FunctionType[][] allow = {{FunctionType.INT_FUNCTION, FunctionType.RESOURCE_FUNCTION}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params) throws ParameterException
	{
		this.intFunc      = (IntFunction)params[0];
		this.resourceFunc = (ResourceFunction)params[1];

		try
		{
			Resource resource = resourceFunc.getResource();
			ResourceIterator it = resource.keys();
			ByteArray key = null;
			Map temp = new HashMap();
			while((key = it.next()) != null)
			{
				ByteArray value = resource.getValue(key);
				int k = key.parseInt();
				Key v = new StringLiteral(value.toString()).getKey();
				temp.put(v, new Integer(k));
			}

			skeys = new SwitchKey[Primes.selectPrime((int)(temp.size()*1.25))];
			this.size = skeys.length;

			Iterator it2 = temp.keySet().iterator();
			while(it2.hasNext())
			{
				Key k = (Key)it.next();
				Integer i = (Integer)temp.get(k);
				load(i,k);
			}
		}
		catch(Exception e)
		{
			throw new ParameterException(this, "Error loading switch",  e);
		}
	}

	private void load(Integer i, Key key) throws Exception
	{
		SwitchKey sk = new SwitchKey();
		sk.key = key;
		sk.num = i.intValue();
		int index = i.intValue();
		index = this.size % index;
		SwitchKey sk1 = null;
		SwitchKey lsk = null;
		for(sk1 = skeys[index]; sk1 != null; lsk = sk1, sk1 = sk1.link);

		if(lsk == null)
		{
			skeys[index] = sk;
		}
		else
		{
			lsk.link = sk;
		}
	}

	public Key getKey() throws FunctionException
	{
		int val = this.intFunc.getInt();
		int	index = this.size % val;

		for(SwitchKey sk = skeys[index]; sk != null; sk = sk.link)
		{
			if(sk.num == val)
			{
				return sk.key;
			}
		}

		return nullKey;
	}

	private class SwitchKey
	{
		public Key key;
		public int num;
		public SwitchKey link;
	}

	public void destroy() throws FunctionException
	{
		this.skeys = null;
	}
}