/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.*;
import amberstone.transaction.*;

public class TransactionFunction extends Function implements KeyFunction, BooleanFunction, IntFunction, LongFunction, Dimension
{

	private Key name;
	private Key nullKey = new StringKey("");
	private static TransactionTrackerTemplate currentTracker;

	private TransactionTrackerTemplate transactionTrackerTemplate;

	public static void setTracker(TransactionTrackerTemplate tracker)
	{
		currentTracker = tracker;
	}

	private static final FunctionType[][] allow = {{},{FunctionType.STRING_LITERAL}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public String getDimension()
	{
		return name.toString();
	}

	public void params(Function[] params) throws ParameterException
	{
		if(currentTracker == null)
		{
			throw new ParameterException(this, "Transaction function must be called within a transaction.");
		}
		this.transactionTrackerTemplate = currentTracker;
		if(params.length == 1)
		{
			this.name = ((StringLiteral)params[0]).getKey();
			if(!currentTracker.hasProp(name.toString()))
			{
				throw new ParameterException(params[0], "Undefined transaction property: "+this.name);
			}
		}
	}

	public Key getKey()
	{
		Prop prop = this.transactionTrackerTemplate.transaction.getProp(name);
		if(prop == null)
		{
			return nullKey;
		}
		else
		{
			return prop.getKey();
		}
	}

	public int getInt()
	{
		Prop prop = this.transactionTrackerTemplate.transaction.getProp(name);
		if(prop == null)
		{
			return 0;
		}
		else
		{
			return prop.getInt();
		}
	}

	public long getLong()
	{
		Prop prop = this.transactionTrackerTemplate.transaction.getProp(name);
		if(prop == null)
		{
			return 0;
		}
		else
		{
			return prop.getLong();
		}
	}

	public Transaction getTransaction()
	{
		return this.transactionTrackerTemplate.transaction;
	}

	public String toString()
	{
		Prop prop = this.transactionTrackerTemplate.transaction.getProp(name);
		if(prop == null)
		{
			return null;
		}
		else
		{
			return prop.getKey().toString();
		}
	}

	public boolean getBoolean()
	{
		Prop value = this.transactionTrackerTemplate.transaction.getProp(name);
		if(value == null)
		{
			return false;
		}
		else
		{
			return true;
		}
	}
}