/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.lang;

import amberstone.data.ByteArray;
import amberstone.data.Key;

public class When extends Function
{
	private BooleanFunction boolFunc;
	private KeyFunction keyFunc;
	private IntFunction intFunc;
	private LongFunction longFunc;
	private BytesFunction bytesFunc;

	private static final FunctionType[][] allow = {{FunctionType.BOOLEAN_FUNCTION, FunctionType.INT_FUNCTION},
		                                           {FunctionType.BOOLEAN_FUNCTION, FunctionType.KEY_FUNCTION},
                                                   {FunctionType.BOOLEAN_FUNCTION, FunctionType.LONG_FUNCTION},
												   {FunctionType.BOOLEAN_FUNCTION, FunctionType.BYTES_FUNCTION}};


	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params)
	{
		this.boolFunc = (BooleanFunction)params[0];
		if(params[1] instanceof KeyFunction)
		{
			this.keyFunc = (KeyFunction)params[1];
		}

		if(params[1] instanceof IntFunction)
		{
			this.intFunc = (IntFunction)params[1];
		}

		if(params[1] instanceof LongFunction)
		{
			this.longFunc = (LongFunction)params[1];
		}

		if(params[1] instanceof BytesFunction)
		{
			this.bytesFunc = (BytesFunction)params[1];
		}
	}

	public boolean pass() throws FunctionException
	{
		return this.boolFunc.getBoolean();
	}

	public Key getKey() throws FunctionException
	{
		return this.keyFunc.getKey();
	}

	public int getInt() throws FunctionException
	{
		return this.intFunc.getInt();
	}

	public long getLong() throws FunctionException
	{
		return this.longFunc.getLong();
	}

	public ByteArray getBytes() throws FunctionException
	{
		return this.bytesFunc.getBytes();
	}
}