/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/


package amberstone.load;

import java.io.FileOutputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.File;
import java.util.Calendar;
import java.util.GregorianCalendar;

import amberstone.data.DataRecord;
import amberstone.data.ByteArray;

public class DataRecordWriter
{
	private int maxBlockSize;
	private int maxRecordSize;

	private int fileNumber;
	private int partNumber;
	private int blockSize;
	private String outDir;
	private String relativePath;
	private int currentYYYYMMDD;
	private Calendar calendar = new GregorianCalendar();

	public DataRecordWriter(int maxBlockSize,
							int maxRecordSize,
					      	String outDir)
		throws Exception
	{
		this.maxBlockSize = maxBlockSize;
		this.maxRecordSize = maxRecordSize;
		this.outDir = outDir;
	}

	private void open(int YYYY,
					  int MM,
					  int DD,
					  int partNumber,
					  DataRecord dataRecord)
		throws IOException
	{
		int YYYYMMDD = getYYYYMMDD(YYYY, MM, DD);
		this.currentYYYYMMDD = YYYYMMDD;
		this.partNumber = partNumber;
		File dir = new File(outDir+File.separatorChar+Integer.toString(YYYY)+File.separatorChar+pad(MM)+File.separatorChar+pad(DD)+File.separatorChar+partNumber);
		dir.mkdirs();
		dataRecord.setDir(dir);
		dataRecord.reset();
		blockSize = 0;
	}

	private String pad(int i)
	{
		String s = Integer.toString(i);
		if(s.length() == 1)
		{
			return 0+s;
		}

		return s;
	}

	public void writeData(int YYYY, int MM, int DD, int part, DataRecord record)
		throws Exception
	{

		int recYYYYMMDD = getYYYYMMDD(YYYY, MM, DD);

		if(recYYYYMMDD > currentYYYYMMDD)
		{
			if(currentYYYYMMDD > 0)
			{
				close(record);
			}

			open(YYYY,
				 MM,
				 DD,
				 part,
				 record);
		}

		currentYYYYMMDD = recYYYYMMDD;

		int readyToLoad = record.prepareRecord();
		if(readyToLoad < 0 || blockSize+readyToLoad > maxBlockSize)
		{
			close(record);
			open(YYYY,
				 MM,
				 DD,
			     ++partNumber,
			     record);
		}

		blockSize+=record.writeRecord();
	}

	private int getYYYYMMDD(int YYYY, int MM, int DD)
	{
		int YYYYMMDD = (YYYY*10000)+(MM*100)+DD;
		return YYYYMMDD;
	}

	public void close(DataRecord record)
		throws IOException
	{
		record.writeBlock();
	}
}