/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.load;

import amberstone.data.ByteArray;
import amberstone.lang.Nullable;
import amberstone.lang.IntFunction;
import amberstone.lang.LongFunction;
import amberstone.lang.BytesFunction;

public class DelimitedField implements BytesFunction,
					                   IntFunction,
					                   LongFunction,
					                   Nullable,
					                   Comparable
{
	private int column;
	private DelimitedRecord delimitedRecord;
	private ByteArray bytes = new ByteArray();
	private boolean isNull;

	public DelimitedField(int column, DelimitedRecord delimitedRecord)
	{
		this.column = column;
		this.delimitedRecord = delimitedRecord;
	}

	public boolean isNull()
	{
		return isNull;
	}

	public boolean equals(Object o)
	{
		DelimitedField f = (DelimitedField)o;
		if(f.column == column)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	public int compareTo(Object o)
	{
		DelimitedField f = (DelimitedField)o;
		if(f.column == column)
		{
			return 0;
		}
		else if(f.column < column)
		{
			return 1;
		}
		else
		{
			return -1;
		}
	}

	public void setBytes()
	{
		int start = delimitedRecord.getFieldStartIndex(column);
		int end   = delimitedRecord.getFieldEndIndex(column);
		if(start == end)
		{
			bytes.set(delimitedRecord.getBuffer(), 0, 0);
			isNull = true;
		}
		else
		{
			bytes.set(delimitedRecord.getBuffer(), start, end-start);
			isNull = false;
		}
	}

	public ByteArray getBytes()
	{
		return bytes;
	}

	public int getInt()
	{
		return bytes.parseInt();
	}

	public long getLong()
	{
		return bytes.parseLong();
	}
}