/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.load;

import amberstone.data.ByteArray;

public class DelimitedRecord
{
	private int endMarker;
	private DelimitedField[] delimitedFields;

	private DelimitedField[] activeDelimitedFields;

	byte[] buffer;
	int[] markers;

	private int beginIndex;
	private int endIndex;

	public DelimitedRecord(int bufferSize, int numFields)
	{
		this.markers = new int[numFields];
		this.buffer = new byte[bufferSize];
		delimitedFields = new DelimitedField[markers.length];
		for(int i=0; i<delimitedFields.length; i++)
		{
			delimitedFields[i] = new DelimitedField(i, this);
		}
	}

	public int[] getMarkers()
	{
		return this.markers;
	}

	public byte[] getBuffer()
	{
		return this.buffer;
	}

	public DelimitedField getField(int column)
	{
		if(column < this.delimitedFields.length)
		{
			return this.delimitedFields[column];
		}
		else
		{
			return null;
		}
	}

	public void setEndMarker(int endMarker)
	{
		this.endMarker = endMarker;
	}

	public int getEndMarker()
	{
		return this.endMarker;
	}

	public int getEndIndex()
	{
		return this.endIndex;
	}

	public int getBeginIndex()
	{
		return this.beginIndex;
	}

	public void setEndIndex(int i)
	{
		this.endIndex = i;
	}

	public void setBeginIndex(int i)
	{
		this.beginIndex = i;
	}

	public int getFieldStartIndex(int field)
	{
		if(field > endMarker+1)
		{
			return -1;
		}

		if(field == 0)
		{
			return beginIndex;
		}
		else
		{
			int fieldOffset = markers[field-1];
			return fieldOffset+beginIndex+1;
		}
	}

	public int getFieldEndIndex(int field)
	{
		if(field > endMarker+1)
		{
			return -1;
		}

		if(field == endMarker+1)
		{
			return endIndex;
		}
		else
		{
			int fieldOffset = markers[field];
			return (fieldOffset+beginIndex);
		}
	}

	public void setActiveFields(DelimitedField[] activeDelimitedFields)
	{
		this.activeDelimitedFields = activeDelimitedFields;
	}

	public void setBytes()
	{
		for(int i=0; i<activeDelimitedFields.length; i++)
		{
			activeDelimitedFields[i].setBytes();
		}
	}
}