/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.load;

import java.io.InputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

public class DelimitedRecordReader
{
	private InputStream in;
	private int fieldDelim;
	private int lastIndex;
	private int index;
	private int begin;
	private byte[] buffer;
	private int[] markers;
	private DelimitedRecord delimitedRecord;

	public DelimitedRecordReader(byte fieldDelim,
							     DelimitedRecord delimitedRecord)
	{
		this.fieldDelim = fieldDelim;
		this.delimitedRecord = delimitedRecord;
		this.buffer =  delimitedRecord.getBuffer();
		this.markers = delimitedRecord.getMarkers();
	}

	public void open(File f)
		throws IOException
	{
		this.in = new FileInputStream(f);
		fill(0);
	}

	public void close()
		throws Exception
	{
		this.lastIndex = this.begin = 0;
		in.close();
	}

	public boolean readRecord() throws ParseException
	{
		begin = index;
		int markerIndex = -1;
		this.delimitedRecord.setBeginIndex(begin);
		try
		{
			int data = 0;

			while(index <= lastIndex || fill(begin) > -1)
			{
				data = buffer[index];

				if(data == fieldDelim)
				{
					markers[++markerIndex] = index-begin;
					index++;
				}
				else if(data > 13)
				{
					index++;
				}
				else if(data > 9)
				{
					if(index == begin)
					{
						index++;
						begin = index;
						this.delimitedRecord.setBeginIndex(begin);
					}
					else
					{
						this.delimitedRecord.setEndIndex(index);
						this.delimitedRecord.setEndMarker(markerIndex);
						this.delimitedRecord.setBytes();
						index++;
						return true;
					}
				}
				else
				{
					index++;
				}
			}
		}
		catch(Exception e)
		{
			throw new ParseException(markerIndex, e);
		}

		return false;
	}

	private int fill(int begin)
		throws IOException
	{
		int copyBytes = index-begin;

		if(copyBytes > 0)
		{
			System.arraycopy(buffer,
			                 begin,
			                 buffer,
			                 0,
			                 copyBytes);

			this.index = copyBytes;
		}
		else
		{
			this.index = 0;
		}

		this.begin = 0;
		this.delimitedRecord.setBeginIndex(0);

		int bytesFilled = in.read(buffer, this.index, (this.buffer.length-1)-index);
		this.lastIndex = bytesFilled+index-1;
		return bytesFilled;
	}
}