/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.merge;

import amberstone.system.*;
import amberstone.file.FilePath;
import amberstone.merge.*;

import java.io.*;
import java.util.*;
import java.math.*;

public class Rollup
{
	private Config config;

	public Rollup(Config config)
	{
		this.config = config;
	}

	public void rollup() throws Exception
	{
		String s = Config.AMBERSTONE_HOME+File.separatorChar+"aggregates"+File.separatorChar+"cubes";
		File in = new File(s);
		walkDays(in);
	}

	public void walkDays(File in) throws Exception
	{
		File[] files = in.listFiles();
		for(int i=0; i<files.length; i++)
		{
			if(files[i].isDirectory())
			{
				walkHours(files[i]);
			}
		}
	}

	public void walkHours(File in) throws Exception
	{
		File[] files = in.listFiles();
		for(int i=0; i<files.length; i++)
		{
			if(files[i].isDirectory())
			{
				rollup(files[i], in);
			}
		}
	}

	private void rollup(File hour, File day) throws Exception
	{
		File[] summaries = getFiles(hour);
		String dayName = day.getName();
		int hourCol = 0;
		if(dayName.length() == 8)
		{
			hourCol = 4;
		}
		else
		{
			hourCol = 3;
		}

		for(int i=0; i<summaries.length; i++)
		{
			BufferedReader in = new BufferedReader(new FileReader(summaries[i]));
			String name = summaries[i].getName();
			String[] parts = name.split("\\.");
			String newName = parts[0]+"."+parts[1]+"."+parts[2]+"."+hour.getName()+".tsv";
			PrintWriter out = new PrintWriter(new FileWriter(new File(day, newName)));
			String line = null;
			while((line = in.readLine()) != null)
			{
				String[] lparts = line.split("	");
				out.println(dropJoin(lparts[0].split("::"), hourCol)+"	"+lparts[1]);
			}

			in.close();
			out.close();
		}
	}

	private String dropJoin(String parts[], int drop)
	{
		StringBuffer buf = new StringBuffer();
		for(int i=0; i<parts.length; i++)
		{
			if(i != drop)
			{
				if(i > 0)
				{
					buf.append("::");
				}

				buf.append(parts[i]);
			}
		}

		return buf.toString();
	}

	private File[] getDirs(File root)
	{
		ArrayList<File> dirs = new ArrayList();
		File[] files = root.listFiles();
		for(int i=0;i<files.length; i++)
		{
			if(files[i].isDirectory())
			{
				dirs.add(files[i]);
			}
		}

		return dirs.toArray(new File[dirs.size()]);
	}

	private File[] getFiles(File root)
	{
		ArrayList<File> dirs = new ArrayList();
		File[] files = root.listFiles();
		for(int i=0;i<files.length; i++)
		{
			if(files[i].getName().endsWith(".tsv"))
			{
				dirs.add(files[i]);
			}
		}

		return dirs.toArray(new File[dirs.size()]);
	}
}