/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.read;

import amberstone.data.Field;
import amberstone.lang.*;
import amberstone.file.FileStat;

import java.util.List;
import java.io.File;

public class DefaultPrinter implements Printer
{
	public void open()
	{

	}

	public void close()
	{

	}

	public void print(Record record) throws Exception
	{
		Col[] fields = record.getFields();
		Selector selector = record.getSelector();

		if(fields.length > 1)
		{
			System.out.println("");
		}

		int width = getMaxWidth(fields);
		for(int i=0; i<fields.length; i++)
		{
			Col field = fields[i];
			String name = field.getName();
			String value = field.toString();
			if(fields.length > 1)
			{
				System.out.print(pad(name, width)+" : ");
			}

			System.out.println(value);
		}
	}

	private String pad(String s, int width)
	{
		while(s.length() < width)
		{
			s = s+" ";
		}
		return s;
	}

	private int getMaxWidth(Col[] reads)
	{
		int width = 0;
		for(int i=0; i<reads.length; i++)
		{
			Col read = reads[i];
			String name = read.getName();
			if(name.length() > width)
			{
				width = name.length();
			}
		}

		return width;
	}
}