/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.read;

import amberstone.summary.Range;
import amberstone.summary.RangeFilter;

import java.io.*;
import java.util.*;

public class MonthDir
{
	private RangeFilter filter;
	private Comparator comp;

	private File dir;
	private String prefix;

	public MonthDir(File dir, RangeFilter filter, Comparator comp)
	{
		File p = dir.getParentFile();
		prefix = p.getName()+dir.getName();
		this.dir = dir;
		this.filter = filter;
		this.comp = comp;
	}

	public List<DayDir> getDays()
	{
		Set ranges = filter.getRanges();
		Iterator i = ranges.iterator();

		DirFilter dayFilter = new DirFilter(prefix);

		while(i.hasNext())
		{
			Range range = (Range)i.next();
			int beginYYYYMMDD = range.getBegin();
			int endYYYYMMDD   = range.getEnd();

			dayFilter.addRange(new Range(beginYYYYMMDD, endYYYYMMDD));
		}

		File[] dirs = dir.listFiles(dayFilter);


		Arrays.sort(dirs, comp);
		List<DayDir> days = new ArrayList();
		for(int y=0; y<dirs.length; y++)
		{
			DayDir dayDir = new DayDir(dirs[y], comp);
			days.add(dayDir);
		}

		return days;
	}

	public String toString()
	{
		return dir.getPath();
	}
}