/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.read;

import amberstone.file.*;
import amberstone.summary.Range;
import amberstone.summary.RangeFilter;
import amberstone.data.*;
import amberstone.lang.*;
import amberstone.time.TimeSeries;
import java.io.*;
import java.util.Set;
import java.util.List;
import java.util.ArrayList;
import java.util.LinkedList;

public class Select extends Function implements Selector
{

	private Read[] reads;
	private BooleanFunction booleanFunc = new BooleanLiteral(true);

	private FileHandler fileHandler;
	private RangeFilter filter;

	private LinkedList stack = new LinkedList();
	private FileStat ready;
	private int readyIndex;
	private int readySize;
	private int recLength;
	private ByteArray bytes = new ByteArray();
	private DataRecord dataRecord;
	private TimeSeries timeSeries;
	private int recordsPerBlock;
	private int startYYYYMMDD;
	private int endYYYYMMDD;
	private	int exportNum = 0;
	private Col[] fields;
	private String[] names;


	public FunctionType[][] strict()
	{
		return null;
	}

	public void params(Function[] params)
	{
		fields = new Col[params.length];
		for(int i=0; i<params.length; i++)
		{
			fields[i] = (Col)params[i];
		}
	}

	public void open(int blockSize,
	                 String dir,
	                 DataRecord dataRecord,
	                 int startYYYYMMDD,
	                 int endYYYYMMDD) throws FunctionException
	{

		this.dataRecord = dataRecord;
		ArrayList list = new ArrayList();
		dataRecord.initRead(blockSize);
		ByteArray bytes = new ByteArray();
		this.fileHandler = new FileHandler();

		int min = startYYYYMMDD;
	    int max = endYYYYMMDD;

		Range range = new Range(min, max);
		this.filter = new RangeFilter();
		this.filter.addRange(range);

		File rootDir = new File(dir);
		List<YearDir> years = YearDir.getYears(rootDir, filter, new DirDesc());

		for(int y=0; y<years.size(); y++)
		{
			stack.addLast(years.get(y));
		}
	}

	private int readNextBlock() throws IOException
	{
		while(true)
		{
			if(stack.size() == 0)
				return -1;

			Object o = stack.removeLast();
			if(o instanceof YearDir)
			{
				YearDir yearDir = (YearDir)o;
				List<MonthDir> months = yearDir.getMonths();
				for(int i=0; i<months.size(); i++)
				{
					stack.addLast(months.get(i));
				}
			}
			else if(o instanceof MonthDir)
			{
				MonthDir monthDir = (MonthDir)o;
				List<DayDir> days = monthDir.getDays();
				for(int i=0; i<days.size(); i++)
				{
					stack.addLast(days.get(i));
				}
			}
			else if(o instanceof DayDir)
			{
				DayDir dayDir = (DayDir)o;
				List<Block> blocks = dayDir.getBlocks();
				for(int i=0; i<blocks.size(); i++)
				{
					stack.addLast(blocks.get(i));
				}
			}
			else if(o instanceof Block)
			{
				Block block = (Block)o;
				this.exportNum = parseDay(block.getDir());
				this.dataRecord.setDir(block.getDir());
				return this.dataRecord.readBlock();
			}
		}
	}

	int num = 0;
	int rnum = 0;

	public Record next() throws FunctionException
	{
		try
		{
			while(true)
			{
				if(!dataRecord.readRecord())
				{
					if(readNextBlock() == -1)
					{
						return null;
					}
				}
				else
				{
					++num;
					LineNumber.LINE_NUMBER = num;

					if(booleanFunc.getBoolean())
					{
						rnum++;
						RecordNumber.RECORD_NUMBER = rnum;

						Record rec = new Record(this, fields);
						rec.setExport(Integer.toString(this.exportNum));
						return rec;
					}
				}
			}
		}
		catch(FunctionException fe)
		{
			throw fe;
		}
		catch(Exception e)
		{
			throw new FunctionException("Error in read", this, e);
		}
	}

	private int parseDay(File block)
	{
		File day = block.getParentFile();
		int dd = Integer.parseInt(day.getName());
		File month = day.getParentFile();
		int mm = Integer.parseInt(month.getName());
		File year = month.getParentFile();
		int yyyy = Integer.parseInt(year.getName());

		int YYYYMMDD = (yyyy*10000)+(mm*100)+dd;
		return YYYYMMDD;
	}
}