/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.resources;

import amberstone.data.ByteArray;

import java.util.Map;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Set;
import java.io.IOException;

public class MemoryResource implements Resource
{
	private Map<ByteArray, ByteArray> index;
	private Map<ByteArray, ByteArray> reverseIndex;
	private Resource resource;

	public MemoryResource(Resource resource) throws IOException
	{
		this.index = new HashMap();
		this.reverseIndex = new HashMap();
		ResourceIterator it = resource.keys();
		for(ByteArray key = it.next(); key != null; key = it.next())
		{
			ByteArray value = resource.getValue(key);
			index.put(key, value);
			reverseIndex.put(value, key);
		}
	}

	public void clear()
	{
		this.index = null;
		this.reverseIndex = null;
	}

	public String getName()
	{
		return resource.getName();
	}

	public ResourceIterator keys()
	{
		return new MemoryResourceIterator(index.keySet().iterator());
	}

	public ResourceIterator values()
	{
		return new MemoryResourceIterator(reverseIndex.keySet().iterator());
	}

	public String get(String key)
	{
		ByteArray value = getValue(new ByteArray(key));
		if(value == null)
		{
			return null;
		}
		else
		{
			return value.toString();
		}
	}

	public ByteArray getValue(ByteArray key)
	{
		return index.get(key);
	}

	public ByteArray getKey(ByteArray value)
	{
		return reverseIndex.get(value);
	}
}