/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.resources;

import amberstone.lang.Function;
import amberstone.lang.FunctionType;
import amberstone.lang.ParameterException;
import amberstone.lang.ResourceFunction;
import amberstone.data.ByteArray;
import java.io.IOException;

public class ReverseResource extends Function implements Resource, ResourceFunction
{
	private Resource resource;

	private static final FunctionType[][] allow = {{FunctionType.RESOURCE_FUNCTION}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params) throws ParameterException
	{
		try
		{
			this.resource = ((ResourceFunction)params[0]).getResource();
		}
		catch(Exception e)
		{

		}
	}

	public String getName()
	{
		return resource.getName();
	}

	public Resource getResource()
	{
		return this;
	}

	public String get(String value) throws IOException
	{
		ByteArray key = (ByteArray)getValue(new ByteArray(value));
		if(key != null)
		{
			return key.toString();
		}

		return null;
	}

	public ResourceIterator keys() throws IOException
	{
		return resource.values();
	}

	public ResourceIterator values() throws IOException
	{
		return resource.keys();
	}

	public ByteArray getValue(ByteArray key) throws IOException
	{
		return resource.getKey(key);
	}

	public ByteArray getKey(ByteArray value) throws IOException
	{
		return resource.getValue(value);
	}
}