/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.summary;

import java.io.File;
import java.io.FileFilter;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

public class RangeFilter
	implements FileFilter
{
	private Set<Range> ranges = new HashSet();

	public void addRange(Range range)
	{
		this.ranges.add(range);
	}

	public Set getRanges()
	{
		return this.ranges;
	}

	public boolean accept(File f)
	{
		String name = f.getName();
		String[] parts = name.split("\\.");
		int num = Integer.parseInt(parts[0]);

		Iterator<Range> it = ranges.iterator();
		while(it.hasNext())
		{
			Range range = it.next();
			if(range.inRange(num))
			{
				return true;
			}
		}

		return false;
	}

	public String toString()
	{
		StringBuilder buf =  new StringBuilder();
		Iterator<Range> it = ranges.iterator();
		while(it.hasNext())
		{
			Range range = it.next();
			buf.append(range.toString());
			if(it.hasNext())
			{
				buf.append(",");
			}
		}
		return buf.toString();
	}
}