/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.system;

import amberstone.file.*;
import java.io.*;

public class ArchiveCommand implements Command
{
	private boolean valid;
	private String index;

	public ArchiveCommand(String args[])
	{
		if(args.length == 2)
		{
			this.index = args[1];
			valid = true;
		}
		else
		{
			valid = false;
		}
	}

	public boolean valid()
	{
		return valid;
	}

	public String usage()
	{
		return "archive [index]";
	}

	public void run(Config config)
	{
		try
		{
			String loadQueue   = Config.AMBERSTONE_HOME+File.separatorChar+"storage"+File.separatorChar+index+File.separatorChar+"load-queue";
			String loadArchive = Config.AMBERSTONE_HOME+File.separatorChar+"storage"+File.separatorChar+index+File.separatorChar+"load-archive";
			File rootDir = new File(loadQueue);
			FileHandler fileHandler = new FileHandler();
			FileStat[] topLevelFiles = fileHandler.getFiles(rootDir, null, false); // get the top level files but don't recurse
			moveFiles(topLevelFiles, loadQueue, loadArchive);
		}
		catch(Exception e)
		{
			System.out.println("Error in archive command:");
			System.out.println(e.getMessage());
			Trace trace = new Trace();
			trace.trace(e, "list", config.getLogsHome());
		}
	}

	private void moveFiles(FileStat[] files, String inDir, String destDir)
	{
		System.out.println("Archiving files:");
		for(int i=0; i<files.length;i++)
		{
			FileStat fstat = files[i];
			File file = fstat.file;
			String path = file.getPath();
			path = path.replace(file.getName(), "");
			path = path.replace(inDir, destDir);
			File newPath = new File(path+file.getName().substring(0,4)+File.separatorChar+file.getName().substring(4, 6)+File.separatorChar+file.getName().substring(6, 8));
			newPath.mkdirs();
			File newFile = new File(newPath, file.getName());
			if(file.renameTo(newFile))
			{
				System.out.println("Archived file:"+newFile.getName());
			}
			else
			{
				System.out.println("Archived failed file:"+newFile.getPath());
			}
		}
	}

	public boolean needsResources()
	{
		return false;
	}
}