/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.system;

import amberstone.file.*;
import amberstone.dictionary.Dictionary;
import amberstone.dictionary.DiskDictionary;
import java.io.File;
import java.io.FileFilter;
import java.util.Map;
import java.util.Iterator;

public class DropCommand implements Command
{
	private String args[];

	public DropCommand(String[] args)
	{
		this.args = args;
	}

	public boolean valid()
	{
		if(args.length == 2 || args.length == 3)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	public String usage()
	{
		return "drop [table] <column>";
	}

	public Selector getSelector()
	{
		return null;
	}

	public boolean needsResources()
	{
		return false;
	}

	public void run(Config config)
	{
		try
		{
			String dir = Config.AMBERSTONE_HOME+File.separatorChar+"storage"+File.separatorChar+args[1];
			if(args.length == 2)
			{
				drop(new File(dir));
			}
			else
			{
				dropCol(new File(dir, "table"), args[2]);
			}
		}
		catch(Exception e)
		{
			System.out.println("Error in list command:");
			System.out.println(e.getMessage());
			Trace trace = new Trace();
			trace.trace(e, "list", config.getLogsHome());
		}
	}

	public void drop(File dir)
	{
		File[] files = dir.listFiles();
		for(int i=0; i<files.length; i++)
		{
			if(files[i].isFile())
			{
				files[i].delete();
			}
			else
			{
				drop(files[i]);
			}
		}
		dir.delete();
	}

	public void dropCol(File dir, String col)
	{
		File[] years = dir.listFiles();
		for(int y=0; y<years.length; y++)
		{
			File[] months = years[y].listFiles();
			for(int m=0; m<months.length; m++)
			{
				File[] days = months[m].listFiles();
				for(int d=0; d<days.length; d++)
				{
					File[] blocks = days[d].listFiles();
					for(int b=0; b<blocks.length; b++)
					{
						File[] cols = blocks[b].listFiles();
						for(int c=0; c<cols.length; c++)
						{
							String name = cols[c].getName().split("\\.")[0];
							if(name.equals(col))
							{
								cols[c].delete();
							}
						}
					}
				}
			}
		}
	}
}