/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.system;

import amberstone.data.ByteArray;
import amberstone.dictionary.Dictionary;
import amberstone.dictionary.DiskDictionary;
import amberstone.resources.Resource;
import amberstone.resources.ResourceIterator;
import amberstone.resources.DiskResource;

import java.util.Map;

public class DumpCommand implements Command
{
	private Config config;
	private String[] args;

	public DumpCommand(String[] args)
	{
		this.args = args;
	}

	public boolean valid()
	{
		if(args.length == 3 || args.length == 4)
		{
			if(args[1].equals("resource") || args[1].equals("dictionary"))
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		else
		{
			return false;
		}
	}

	public String usage()
	{
		return "-dump <dictionary|resource> <name> [key]";
	}

	public Selector getSelector()
	{
		return null;
	}

	public boolean needsResources()
	{
		return true;
	}

	public void run(Config config)
	{
		this.config = config;
		try
		{
			String type = args[1];
			String name = args[2];
			String key = null;

			if(args.length == 4)
			{
				key = args[3];
			}

			if(type.equals("dictionary"))
			{
				System.out.println("dump dict");
				dumpDictionary(name, key);
			}
			else if(type.equals("resource"))
			{
				dumpResource(name, key);
			}
		}
		catch(Exception e)
		{

		}
	}

	public void dumpDictionary(String name, String key) throws Exception
	{
		Map dicts = config.getDictionaries();
		DiskDictionary dict = (DiskDictionary)dicts.get(name);

		if(key == null)
		{
			ResourceIterator ri = dict.terms();
			ByteArray term = null;
			while((term = ri.next()) != null)
			{
				if(!"_SEQ".equals(term.toString()))
				{
					int id = dict.getId(term);
					System.out.println(term.toString()+":"+id);
				}
			}
		}
		else
		{
			int id = dict.getId(new ByteArray(key));
			System.out.println(key+":"+id);
		}
	}

	public void dumpResource(String name, String key) throws Exception
	{
		Map resources = config.getResources();
		DiskResource resource = (DiskResource)resources.get(name);
		if(key == null)
		{
			ResourceIterator ri = resource.keys();
			ByteArray k = null;
			while((k = ri.next()) != null)
			{
				ByteArray value = resource.getValue(k);
				System.out.println(k.toString()+":"+value.toString());
			}
		}
		else
		{
			ByteArray value = resource.getValue(new ByteArray(key));
			System.out.println(key+":"+value.toString());
		}
	}
}