/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.system;

import java.util.GregorianCalendar;
import java.util.Calendar;
import java.io.FileWriter;
import java.io.BufferedWriter;
import java.io.PrintWriter;
import java.io.File;
import java.io.IOException;

public class Logger
{
	private PrintWriter out;
	private Calendar cal = new GregorianCalendar();

	private static long run = System.currentTimeMillis();

	public Logger(String outDir, String name, int mode) throws SystemException
	{
		try
		{
			if(mode == 0)
			{
				out = new PrintWriter(new BufferedWriter(new FileWriter(outDir+File.separatorChar+name+".log", true), 1024));
			}
			else if(mode == 1)
			{
				out = new PrintWriter(new BufferedWriter(new FileWriter(outDir+File.separatorChar+name+".log"), 1024));
			}
			else if(mode == 2)
			{
				File logFile = new File(outDir+File.separatorChar+name+".log");

				if(logFile.exists())
				{



					File renamed = new File(outDir+File.separatorChar+"archive"+File.separatorChar+name+"."+Long.toString(run)+".log");
					logFile.renameTo(renamed);
				}

				out = new PrintWriter(new BufferedWriter(new FileWriter(outDir+File.separatorChar+name+".log"), 1024));
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new SystemException("Exception opening logger:"+outDir+File.separatorChar+name+".log", e);
		}
	}

	public void close()
	{
		try
		{
			out.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	public void timestamp()
	{
		try
		{
			cal.setTimeInMillis(System.currentTimeMillis());

			String year  = str(cal.get(cal.YEAR), 4);
			String month = str(cal.get(cal.MONTH)+1, 2);
			String day   = str(cal.get(cal.DAY_OF_MONTH), 2);

			String hour =  str(cal.get(cal.HOUR_OF_DAY), 2);
			String minute = str(cal.get(cal.MINUTE), 2);
			String second = str(cal.get(cal.SECOND), 2);

			out.print(month);
			out.print("-");
			out.print(day);
			out.print("-");
			out.print(year);
			out.print(":");
			out.print(hour);
			out.print(":");
			out.print(minute);
			out.print(":");
			out.print(second);
			out.print(" ");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	private String str(int num, int stringSize)
	{
		String s = Integer.toString(num);
		while(s.length() < stringSize)
		{
			s = "0"+s;
		}

		return s;
	}

	public void ln()
	{
		try
		{
			out.println("");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	public void log(String message)
	{
		try
		{
			out.println(message);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	public void log(Throwable e)
	{
		try
		{
			e.printStackTrace(out);
		}
		catch(Exception e1)
		{
			e1.printStackTrace();
		}
	}
}