/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.time;

import amberstone.lang.*;
import amberstone.report.*;

import java.util.GregorianCalendar;
import java.util.Calendar;
import java.util.Map;
import java.util.ArrayList;
import java.util.List;


public abstract class TimeSeries extends Function
{
	protected Increment increment;
	protected String name;
	protected int start;
	protected int end;

	protected static GregorianCalendar cal = new GregorianCalendar();

	public static TimeSeries getInstance(String name)
	{
		if(name.equals("days"))
		{
			return new DaySeries();
		}
		else if(name.equals("weeks"))
		{
			return new WeekSeries();
		}
		else if(name.equals("months"))
		{
			return new MonthSeries();
		}
		else if(name.equals("quarters"))
		{
			return new QuarterSeries();
		}
		else if(name.equals("years"))
		{
			return new YearSeries();
		}

		return null;
	}

	public Increment getIncrement()
	{
		return increment;
	}

	public void setStart(int start)
	{
		this.start = start;
	}

	public void setEnd(int end)
	{
		this.end = end;
	}

	public void setName(String name)
	{
		this.name = name;
	}

	public int[] getCols()
	{
		return getCols(getMinYYYYMMDD(), getMaxYYYYMMDD());
	}

	public abstract int granularity();
	public abstract TimeSeries getTimeSeries(int start, int end);
	public abstract int getMaxYYYYMMDD();
	public abstract int getMinYYYYMMDD();
	public abstract int convertYYYYMMDD(int YYYYMMDD);
	public abstract int currentTime();
	public abstract int getTime(int i);
	public boolean incremental()
	{
		return increment != null;
	}

	public abstract int convertTo(TimeSeries convertSeries, int convertValue);

	public int convertToDays(int value)
	{
		return -1;
	}

	public int convertToMonths(int value)
	{
		return -1;
	}

	public int convertToQuarters(int value)
	{
		return -1;
	}

	public int convertToWeeks(int value)
	{
		return -1;
	}

	public int convertToYears(int value)
	{
		return -1;
	}

	public abstract String output(String value);

	public abstract int[] getCols(int beginYYYYMMDD, int endYYYYMMDD);

	protected static int validateAndFormatYear(String year) throws Exception
	{
		if(year.length() != 4)
		{
			throw new Exception("The year must contain 4 digits: "+year);
		}

		int YYYY = Integer.parseInt(year);
		return YYYY;
	}

	protected static int validateAndFormatMonth(String month) throws Exception
	{
		if(month.length() != 2)
		{
			throw new Exception("The month must contain 2 digits: "+month);
		}

		int MM = Integer.parseInt(month);

		if(MM < 1 || MM > 12)
		{
			throw new Exception("The month must be between the values of 1 and 12: " +month);
		}

		return MM;
	}

	protected static int validateAndFormatQuarter(String quarter)
		throws Exception
	{

		if(quarter.length() != 1)
		{
			throw new Exception("The quarter must contain 1 digit: "+quarter);
		}

		int Q = Integer.parseInt(quarter);


		if(Q< 1 || Q > 4)
		{
			throw new Exception("The quarter must be between the values of 1 and 4: " +quarter);
		}

		return Q;
	}


	protected static int validateAndFormatWeek(String week)
		throws Exception
	{
		if(week.length() != 2)
		{
			throw new Exception("The week must contain 2 digits: "+week);
		}

		int WW = Integer.parseInt(week);

		if(WW < 1 || WW > 52)
		{
			throw new Exception("The week must be between the values of 1 and 52:" +week);
		}

		return WW;
	}

	protected static int validateAndFormatDay(String day)
		throws Exception
	{
		if(day.length() != 2)
		{
			throw new Exception("The day must contain 2 digits: "+day);
		}

		int DD = Integer.parseInt(day);


		if(DD < 1 || DD > 31)
		{
			throw new Exception("The day must be between the values of 1 and 31:" +day);
		}

		return DD;
	}
}