/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.time;

import amberstone.lang.*;
import amberstone.report.*;

import java.util.Arrays;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Calendar;

public class YearSeries extends TimeSeries
{

	private boolean incremental;

	private static final FunctionType[][] allow = {{FunctionType.STRING_LITERAL, FunctionType.STRING_LITERAL},
												   {FunctionType.YEAR, FunctionType.YEAR},
												   {FunctionType.STRING_LITERAL, FunctionType.YEAR},
												   {FunctionType.YEAR, FunctionType.STRING_LITERAL}};

	public FunctionType[][] strict()
	{
		return allow;
	}

	public YearSeries()
	{

	}

	public String output(String YYYY)
	{
		return YYYY;
	}

	public YearSeries(Year year)
	{
		this.increment = year;
		start = end = year.getIncrement();
	}

	public int granularity()
	{
		return 5;
	}

	public TimeSeries getTimeSeries(int start, int end)
	{
		YearSeries yearSeries = new YearSeries();
		yearSeries.start = start;
		yearSeries.end = end;
		return yearSeries;
	}

	public void params(Function[] params) throws ParameterException
	{
		try
		{
			start = validateAndFormat(params[0].toString());
		}
		catch(Exception e)
		{
			throw new ParameterException(params[0], "Invalid year format (YYYY): "+params[0].toString(), e);
		}

		try
		{
			end = validateAndFormat(params[1].toString());
		}
		catch(Exception e)
		{
			throw new ParameterException(params[1], "Invalid year format (YYYY): "+params[1].toString(), e);
		}

		if(start > end)
		{
			throw new ParameterException(params[0], "Start year must be less then or equal to end year");
		}

		if(params[1] instanceof Year)
		{
			this.incremental = true;
		}
	}

	public int getTime(int YYYY)
	{
		int MM = 15;
		int DD = 15;
		cal.clear();
		cal.set(YYYY, MM-1, DD, 15, 0, 0);
		return (int)cal.getTimeInMillis()/1000;
	}



	public static int validateAndFormat(String time)
		throws Exception
	{
		int YYYY = validateAndFormatYear(time);
		return YYYY;
	}

	public String toString()
	{
		return "years";
	}

	public int convertYYYYMMDD(int YYYYMMDD)
	{
		return (int)(YYYYMMDD/10000);
	}

	public int convertTo(TimeSeries convertSeries, int convertValue)
	{
		return convertSeries.convertToYears(convertValue);
	}

	public int convertToYears(int YYYY)
	{
		return YYYY;
	}

	public void setRolling(boolean rolling)
	{
		if(rolling)
		{
			cal.setTimeInMillis(System.currentTimeMillis());
			this.end = cal.get(cal.YEAR);
		}
	}

	public int[] getCols(int minYYYYMMDD, int maxYYYYMMDD)
	{
		int minYYYY = (int)(minYYYYMMDD/10000);
		int minMM = (int)((minYYYYMMDD % (minYYYY*10000)) / 100);
		int minDD = minYYYYMMDD % ((minYYYY*10000)+(minMM*100));

		Set<Integer> years = new HashSet();

		cal.set(minYYYY, minMM-1, minDD);
		while(true)
		{
			int testYYYY = cal.get(Calendar.YEAR);
			int testMM = cal.get(Calendar.MONTH);
			int testDD = cal.get(Calendar.DAY_OF_MONTH);

			int testYYYYMMDD = (testYYYY*10000)+((testMM+1)*100)+testDD;

			if(testYYYYMMDD <= maxYYYYMMDD)
			{
				years.add(new Integer(testYYYY));
				cal.add(Calendar.DATE, 1);
			}
			else
			{
				break;
			}
		}

		int[] y = new int[years.size()];

		Iterator<Integer> it = years.iterator();
		int i = -1;
		while(it.hasNext())
		{
			y[++i] = it.next().intValue();
		}

		Arrays.sort(y);

		return y;
	}

	public int currentTime()
	{
		cal.setTimeInMillis(System.currentTimeMillis());
		int year  = cal.get(cal.YEAR);
		return year;
	}


	public int getMaxYYYYMMDD()
	{
		return (end*10000)+1231;
	}

	public int getMinYYYYMMDD()
	{
		return (start*10000)+101;
	}
}