/**
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
**/

package amberstone.transaction;

import amberstone.lang.*;
import amberstone.data.*;

public class CounterProp extends Function implements Prop
{
	private Key name;
	private LongKey lkey = new LongKey();
	private long count;
	private LongFunction longFunc;
	private BooleanFunction filter;

	private static final FunctionType[][] allow = {{FunctionType.LONG_FUNCTION, FunctionType.BOOLEAN_FUNCTION}};

	public CounterProp()
	{

	}

	public CounterProp(Key name, LongFunction longFunc, BooleanFunction filter)
	{
		this.name = name;
		this.longFunc = longFunc;
		this.filter = filter;
	}

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params)
	{
		this.longFunc = (LongFunction)params[0];
		this.filter = (BooleanFunction)params[1];
	}

	public void update() throws FunctionException
	{
		if(filter.getBoolean())
		{
			count += longFunc.getLong();
		}
	}

	public Prop cloneProp(Key _name)
	{
		return new CounterProp(_name, this.longFunc, this.filter);
	}

	public Key getName()
	{
		return this.name;
	}

	public int getInt()
	{
		return (int)count;
	}

	public long getLong()
	{
		return count;
	}

	public Key getKey()
	{
		this.lkey.setLongValue(count);
		return lkey;
	}
}