/**
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
**/

package amberstone.transaction;

import amberstone.lang.*;
import amberstone.data.*;

public class KeyProp extends Function implements Prop
{
	private Key name;
	private Key nullKey = new StringKey("");
	private Key key;
	private KeyFunction keyFunc;
	private int size;
	private boolean set;
	private boolean overwrite;
	private KeyWrapper wrapper;

	private static final FunctionType[][] allow = {{FunctionType.DIMENSION, FunctionType.BOOLEAN_LITERAL}};

	public KeyProp()
	{
	}

	public KeyProp(Key name, KeyFunction keyFunc, boolean overwrite) throws FunctionException
	{
		this.name = name;
		this.keyFunc = keyFunc;
		this.overwrite = overwrite;
		this.wrapper = new KeyWrapper();
	}

	public FunctionType[][] strict()
	{
		return allow;
	}

	public void params(Function[] params)
	{
		this.keyFunc = (KeyFunction)params[0];
		this.overwrite = ((BooleanLiteral)params[1]).getBoolean();
	}

	public Prop cloneProp(Key _name) throws FunctionException
	{
		return new KeyProp(_name, this.keyFunc, this.overwrite);
	}

	public Key getName()
	{
		return this.name;
	}

	public void init() throws FunctionException
	{
		set = false;
		update();
	}

	public void update() throws FunctionException
	{
		if(!set || overwrite)
		{
			Key k = keyFunc.getKey();
			if(!k.isNull())
			{
				set = true;
				key = k.cloneKey();
			}
		}
	}

	public int getInt()
	{
		if(set)
		{
			return key.intValue();
		}
		else
		{
			return -1;
		}
	}

	public long getLong()
	{
		if(set)
		{
			return key.longValue();
		}
		else
		{
			return -1L;
		}
	}

	public Key getKey()
	{
		if(set)
		{
			wrapper.setKey(key);
			return wrapper;
		}
		else
		{
			return nullKey;
		}
	}

	class KeyWrapper implements Key
	{
		private Key k;

		public void setKey(Key k)
		{
			this.k = k;
		}

		public boolean equalsKey(Key key)
		{
			return k.equalsKey(key);
		}

		public Key cloneKey()
		{
			return k;
		}

		public boolean isNull()
		{
			return k.isNull();
		}

		public int getType()
		{
			return k.getType();
		}

		public int intValue()
		{
			return k.intValue();
		}

		public long longValue()
		{
			return k.longValue();
		}

		public ByteArray byteValue()
		{
			return k.byteValue();
		}

		public int hashCode()
		{
			return k.hashCode();
		}

		public String toString()
		{
			return k.toString();
		}
	}
}