/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.transaction;

import amberstone.data.Key;
import amberstone.data.CompoundKey;
import amberstone.lang.StringLiteral;
import amberstone.lang.BooleanFunction;
import amberstone.lang.KeyFunction;
import amberstone.lang.FunctionException;

import java.util.Map;
import java.util.HashMap;

public class Transaction
{
	Transaction forward;
	Transaction back;
	Transaction link;

	Key id;
	private int begin;
	int lastTouched;

	private Prop[] props;
	private Map<Key, Prop> propMap = new HashMap();

	public Key getId()
	{
		return this.id;
	}

	public int getDuration()
	{
		return begin-lastTouched;
	}

	public Prop getProp(Key name)
	{
		return propMap.get(name);
	}

	public void setProps(Prop[] props) throws FunctionException
	{
		this.props = props;

		for(int i=0; i<props.length; i++)
		{
			propMap.put(props[i].getName(), props[i]);
		}
	}

	public Transaction()
	{

	}

	public Transaction(Key id, int time) throws FunctionException
	{
		this.id = id;
		this.begin = time;
	}

	public void update(int time) throws FunctionException
	{
		this.lastTouched = time;
		for(int i=0; i<props.length; i++)
		{
			props[i].update();
		}
	}
}