/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.transaction;

public class TransactionQueue
{
	private Transaction head = new Transaction();
	private Transaction tail = new Transaction();

	public TransactionQueue()
	{
		head.back = tail;
		tail.forward = head;
	}

	public void remove(Transaction t)
	{
		unlink(t);
	}

	public Transaction pop()
	{
		if(head.back != tail)
		{
			Transaction t = head.back;
			unlink(t);
			return t;
		}

		return null;
	}

	public Transaction peekLast()
	{
		if(tail.forward != head)
		{
			return tail.forward;
		}
		else
		{
			return null;
		}
	}

	public void addFirst(Transaction t)
	{
		t.forward = head;
		t.back = head.back;
		head.back.forward = t;
		head.back = t;
	}

	public void moveFirst(Transaction t)
	{
		t.back.forward = t.forward;
		t.forward.back = t.back;
		t.forward = head;
		t.back = head.back;
		head.back.forward = t;
		head.back = t;
	}

	private void unlink(Transaction t)
	{
		t.back.forward = t.forward;
		t.forward.back = t.back;
		t.forward = t.back = null;
	}
}