/*
*
*	Copyright 2012 Joel Bernstein
*
*	Licensed under the Apache License, Version 2.0 (the "License");
*	you may not use this file except in compliance with the License.
*	You may obtain a copy of the License at
*
*	 http://www.apache.org/licenses/LICENSE-2.0
*
*	Unless required by applicable law or agreed to in writing, software
*	distributed under the License is distributed on an "AS IS" BASIS,
*	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*	See the License for the specific language governing permissions and
*	limitations under the License.
*
*/

package amberstone.transaction;

import amberstone.data.Key;
import amberstone.lang.*;
import amberstone.lang.FunctionException;
import amberstone.summary.FileTemplate;
import amberstone.summary.FileNode;

import amberstone.file.FileStat;

import java.util.List;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;

public class TransactionTrackerTemplate implements FileTemplate
{
	private Calendar cal = new GregorianCalendar();
	private KeyFunction id;
	private PropTemplate[] props = new PropTemplate[0];
	private TimeFunction time;
	private int timeout;
	private int size;

	TransactionLRU activeLRU;
	int currentDay;

	private List<FileTemplate> fileTemplates = new ArrayList();

	public Transaction transaction;

	public TransactionTrackerTemplate(KeyFunction id,
									  TimeFunction time,
								      int timeout,
								      int size)
	{
		this.id = id;
		this.time = time;
		this.timeout = timeout;
		this.size = size;
	}

	public boolean hasProp(String name)
	{
		for(int i=0; i<props.length; i++)
		{
			if(props[i].getName().toString().equals(name))
			{
				return true;
			}
		}
		return false;
	}

	public int fileTemplateCount()
	{
		return this.fileTemplates.size();
	}

	public String getSelectorClass()
	{
		return null;
	}

	public String getName()
	{
		return null;
	}

	public void setProps(PropTemplate[] props)
	{
		this.props = props;
	}

	public FileNode getFileNode()
	{
		FileNode[] fileNodes = new FileNode[fileTemplates.size()];

		for(int i=0; i<fileNodes.length; i++)
		{
			fileNodes[i] = fileTemplates.get(i).getFileNode();
		}

		if(activeLRU == null)
		{
			TransactionTracker t = new TransactionTracker(this, id, time, timeout, size);
			t.setFileNodes(fileNodes);
			t.setProps(props);
			return t;
		}
		else
		{
			TransactionTracker t = new TransactionTracker(this, id, time, timeout, activeLRU);
			t.setFileNodes(fileNodes);
			t.setProps(props);
			return t;
		}
	}

	public List<FileTemplate> getFileTemplates()
	{
		return this.fileTemplates;
	}

	public void addFileTemplate(FileTemplate fileTemplate)
	{
		this.fileTemplates.add(fileTemplate);
	}
}